package manual2;

/**  
 * NAME: ImageDemo10g.java
 *   
 * DESCRIPTION:  ImageServlet is a Java Servlet that demonstrates the use
 * of interMedia Java Classes for Servlets and JSPs by implementing a
 * simple photo album application.
 * 
 * NOTES: See the README.txt file for information on how to build and
 * run the demo. Former (c) Oracle Corp.
 */

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import java.sql.*;
import oracle.ord.im.*;
import oracle.sql.*;
import oracle.jdbc.driver.*;
import javax.naming.InitialContext;
import javax.sql.DataSource;
import javax.naming.NamingException;

 /******************************************************************************
  * ImageServlet is a Java Servlet that demonstrates the use of
  * interMedia Java Classes for Servlets and JSPs by implementing a
  * simple photo album application.
  *****************************************************************************/
public class ImageDemo10g extends HttpServlet
{
  // TODO: nastavit login/password
  //
  // Connection information:
  // - You must select a connection method and supply the necessary
  //   connection information.
  // - You may optionally change the user name/password to connect to a
  //   schema other than scott/tiger.
  //
  private final static String JDBC_CONNECT_STRING = "jdbc:oracle:thin:@pcuifs1.fit.vutbr.cz:1521:stud";
  private final static String JDBC_USER_NAME = "login";
  private final static String JDBC_PASSWORD  = "password";
  // A Java Stack used to implement a simple connection pooling mechanism
  private static Stack connStack = new Stack();

  // Flag indicating if the JDBC driver has been loaded.
  private static boolean driverLoaded = false;


  /*****************************************************************************
   * Servlet initialization method.
   */
  public void init( ServletConfig config ) throws ServletException
  {
    super.init(config);
  }

  /*****************************************************************************
   * Process an HTTP Get request. The following requests are processed by
   * this method:
   * - media: retrieve a full-size or thumb-nail image
   * - new: display a form allowing a new entry to be uploaded
   * - view: view full-size image
   * - [default]: view photo album and thumb-nail images
   */
  public void doGet( HttpServletRequest request, HttpServletResponse response )
    throws ServletException, IOException
  {
    Connection conn = null;

    //
    // Use a try-block to ensure that JDBC connections are always returned
    // to the pool.
    //
    try
    {
      //
      // Get a JDBC connection from the pool
      //
      conn = getTheConnection();

      //
      // Instantiate an ImageRequest object to process the request.
      //

      ImageRequest albumRequest = new ImageRequest( conn, request, response );

      //
      // Figure out what to do based on query string parameters.
      //
      String view_media = request.getParameter( "view_media" );
      if ( view_media != null )
      {
        //
        // Deliver a full-size or thumb-nail image to the browser.
        //
        albumRequest.viewMedia( view_media );
        return;
      }

      // TODO: tudy dale
      else if ( request.getParameter( "view_form" ) != null )
      {
        //
        // Display the HTML upload form.
        //
        albumRequest.viewQueryForm();
      }
      else if ( request.getParameter( "view_load" ) != null )
      {
          // TODO: odtud...
          // Display the HTML Image upload form.
          //
        albumRequest.viewUploadForm();
      }
      else if ( request.getParameter( "view_entry" ) != null )
      {
        //
        // Display full-size photo image.
        //
        albumRequest.viewPhoto();
      }
      else if ( request.getParameter( "view_add" ) != null )

      {
        // form for adding annotations
        albumRequest.viewAddAnnForm();
      }
      else if ( request.getParameter( "view_rotate" ) != null )
      {
        // Process and display rotated image
        albumRequest.viewRotateMedia();
      }
      else if ( request.getParameter( "view_contrast" ) != null )
      {
        // Process and display rotated image
       albumRequest.viewContrastMedia();
     }
      else if ( request.getParameter( "view_brightness" ) != null )
      {
        // Process and display rotated image
        albumRequest.viewBrightnessMedia();
      }
      else
      {
        // TODO: tady najdete zadani...
        // Display album contents with thumb-nails by default.
        //
        albumRequest.viewAlbum();
      }
    }
    catch ( SQLException e )
    {
      //
      // Log what went wrong.
      //
      e.printStackTrace( System.out );

      //
      // Turn SQL exceptions into ServletExceptions.
      //
      throw new ServletException( e.toString() );
    }
    finally
    {
      //
      // If we have a JDBC connection, then return it to the pool.
      //
      freeConnection( conn );
    }
  }


  /*****************************************************************************
   * Process an HTTP Post request to upload a new info. into the database.
   */
  public void doPost( HttpServletRequest request, HttpServletResponse response )
    throws ServletException, IOException
  {
    Connection conn = null;

    //
    // Use a try-block to ensure that JDBC connections are always returned
    // to the pool.
    //
    try
    {
      //
      // Get a JDBC connection from the pool
      //
      conn = getTheConnection();

      //
      // Instantiate an ImageRequest object to process the request
      //
      ImageRequest albumRequest = new ImageRequest( conn, request, response );

      //
      // Create an OrdHttpUploadFormData object.
      //
      OrdHttpUploadFormData formData = new OrdHttpUploadFormData( request );

      //
      // Parse the multipart/form-data message.
      //
      formData.parseFormData();

      // TODO: tudy dale 2
      // Query image table
      //
      if ( formData.getParameter( "query_table" ) != null )
      {
        albumRequest.queryImages(formData);
      }

      // TODO: tudy na chybu
      // TODO: odtud... 2
      // Insert the photo into the album.
      if ( formData.getParameter( "insert_image" ) != null )
      {
        albumRequest.insertNewPhoto(formData);
      }

      if ( formData.getParameter( "add_ann" ) != null )
      {
        albumRequest.addAnnotations(formData);
      }

    }
    catch ( SQLException e )
    {
      //
      // Log what went wrong.
      //
      e.printStackTrace( System.out );

      //
      // Turn SQL exceptions into ServletExceptions.
      //
      throw new ServletException( e.toString() );
    }
    finally
    {
      //
      // If we have ba JDBC connection, then return it to the pool.
      //
      freeConnection( conn );
    }
  }


  /*****************************************************************************
  * Get Servlet information
  */
  public String getServletInfo()
  {
    return "interMedia Java Servlet Demo";
  }
  /**
   * The getTheConnection method implements a simple JDBC connection pool using
   * a Java Stack to hold the set of available connections. If the stack is
   * empty, then getTheConnection simply creates a new connection.
   */
  private Connection getTheConnection() throws SQLException
  {
    OracleConnection conn = null;

    //
    // Synchonize on the Stack object. Load the JDBC driver if not yet
    // done. If there's a free connection on the stack, then pop it off
    // the stack and return it to the caller. Otherwise, create a new
    // connection object and call the version compatibility initialization
    // method.
    //
    synchronized( connStack )
    {
      if ( !driverLoaded )
      {
        DriverManager.registerDriver( new oracle.jdbc.driver.OracleDriver() );
        driverLoaded = true;
      }

      if ( connStack.empty() )
      {
        conn = (OracleConnection)DriverManager.getConnection
          ( JDBC_CONNECT_STRING, JDBC_USER_NAME, JDBC_PASSWORD );
        try
        {
          OrdMediaUtil.imCompatibilityInit( conn );
        }
        catch ( Exception e )
        {
          throw new SQLException( e.toString() );
        }
      }
      else
      {
        conn = (OracleConnection)connStack.pop();
      }
    }

    /*
      protected Connection getTheConnection() throws SQLException
      {
        OracleConnection dbConn = null;
        DataSource ds = null;
        try
        {
          InitialContext ic = new InitialContext();
          ds = (DataSource) ic.lookup("jdbc/ImageDemo");
          dbConn = (OracleConnection) ds.getConnection();
          dbConn.setAutoCommit(true);
          return dbConn;
        }

        catch (NamingException e)
        {
          throw new SQLException(e.toString());
        }
        catch (Exception e)
        {
          if (dbConn!=null && !dbConn.isClosed()) dbConn.close();
          throw new SQLException(e.toString());
        }
      }
      */

    //
    // Enable auto-commit by default.
    //
    conn.setAutoCommit( true );

    return conn;
  }

  /*****************************************************************************
  * The freeConnection method simply returns a JDBC connection to the pool.
  */
  private void freeConnection( Connection conn )
  {
    //
    // Synchonize on the Stack object, then push the connection onto the
    // stack.
    //
    if ( conn != null )
    {
      synchronized( connStack )
      {
        connStack.push( conn );
      }
    }
  }
}



 /******************************************************************************
 * The ImageRequest class is responsible for the actual processing of
 * an HTTP GET or POST request.
 ******************************************************************************/
class ImageRequest
{
  //
  // Request execution information.
  //
  private Connection conn;
  private HttpServletRequest request;
  private HttpServletResponse response;
  private PrintWriter out;
  private String servletUri;

  /*****************************************************************************
   * Constructor
   */
  ImageRequest( Connection conn, HttpServletRequest request, 
                   HttpServletResponse response )
  {
    this.conn = conn;
    this.request = request;
    this.response = response;
    this.out = null;

    //
    // Get the servlet's URI. This shouldn't be necessary, since a servlet
    // should be able to use a self-relative URI to reference itself;
    // however, this doesn't work in JDeveloper, where the URL is always
    // /servlets/servlet.
    //
    servletUri = request.getRequestURI();
  }

  /*****************************************************************************
   * Display the contents of the photo album with thumb-nail images for all
   * the entries.
   */
  void viewAlbum() throws ServletException, IOException, SQLException
  {
    int rowCount = 0;

    //
    // Write a common HTML header.
    //
    printPageHeader();

    //
    // Prepare and execute a SQL statement to fetch the ID, description,
    // expt id and thumb-nail image for all the entries in the album.
    // The thumb-nail image for each entry is used to build the height
    // and width attributes for the image tag.
    //
    PreparedStatement stmt =
      conn.prepareStatement( "select expt_id,description,thumb " +
          " from expt_img_tab_demo order by expt_id" );
    OracleResultSet rset = (OracleResultSet)stmt.executeQuery();

    //
    // Display the thumb-nail images in a table.
    //
    out.println( "<p><table cellpadding=\"3\" cellspacing=\"0\"" +
        " border=\"1\" width=\"100%\">" );
    out.println( "<tr bgcolor=\"#336699\">" );
    //     out.println( "<th><font color=\"#ffffff\">Experiment id</font></th>" );
    out.println( "<th><font color=\"#ffffff\">Image</font></th>" );
    out.println( "<th><font color=\"#ffffff\">Description</font></th></tr>" );

    //
    // For each entry...
    //
    while ( rset.next() )
    {
      //
      // Increment the row count so we know if there are any entries
      // in the album.
      //
      rowCount++;

      //
      // Get the entry ID for building URLs.
      //
      String expt_id = rset.getString( 1 );

      //
      // Start the table row and display the photo description and
      // expt. id.
      //
      out.println( "<tr>" );
      //   out.println( "<td>" + expt_id + "</td>" );

      //
      // Get the thumb-nail OrdImage object from the result set so we
      // can build the height and width attributes in the image tag.
      //
      OrdImage img =
        (OrdImage)rset.getCustomDatum( 3, OrdImage.getFactory() );

      //
      // Display the thumb-nail image as an anchor tag which can be used
      // to display the full-size image. If the image format isn't
      // supported by interMedia, then a thumb-nail wouldn't have been
      // produced when the image was uploaded, so use the text "[view
      // image]" instead of the thumb-nail.
      //
      out.print( "<td><a href=\"" + servletUri +
        "?view_entry=yes&id=" + expt_id + "\">" );

      if ( img.getContentLength() > 0 )
      {
        out.print( "<img src=\"" + servletUri +
          "?view_media=thumb&id=" + expt_id + "\"" +
          " height=" + img.getHeight() +
          " width=" + img.getWidth() +
          " border=1>" );
      }
      else
      {
        out.print( "[view image]" );
      }
      out.println( "</a></td>" );

      out.println( "<td>" + rset.getString( 2 ) + "</td>" );
      out.println( "</tr>" );
    }

    //
    // Display a message if the album is empty
    //
    out.println( "<tr><td colspan=\"3\" align=\"center\">" +
        "<font color=\"#336699\">" );
    if ( rowCount == 0 )
    {
      out.println( "<b><i>The photo album is empty</i></b>" );
    }
    else
    {
      out.println( "<b><i>Select the thumb-nail to view the" +
          " full-size image</i></b>" );
    }
    out.println( "</font></td></tr>" );
    out.println( "</table>" );

    //
    // Display an anchor tag that will display the new entry form.
    //
    printLink( "?view_load=yes", "Insert new image" );
    printLink( "?view_form=yes", "Query image table" );
    
    printMessage("Zadani cviceni 2", 
    "     * Rozchodit aplikaci, aby bylo videt tohle zadani (hotovo) <br>" + 
    "     * Opravit chybu pri vkladani - pouzit sekvenci <br>" + 
    "     * Do definice dat (DDl.sql) pridat ORDSYS.OrdImageSignature <br>" + 
    "     * Do uploadu pridat extrakci rysu (vytvorit signature) <br>" + 
    "     * Do vyhledavaciho formulare nakopirovat uploadovaci (mimo nazvu submit)<br>" + 
    "     * (Volitelne) vytvorit novou tabulku (se sekvenci) pro upload dotazu<br>" + 
    "     * Ulozit prichozi obrazek und signaturu (do zpracovani vyhledavani nakopirovat zpracovani upload + upravit ...) <br>" + 
    "     * Porovnat obrazky <br>" + 
    "     * Zobrazit vysledek dle relevance <br> ");

    //
    // Write the common HTML trailer
    //
    printPageTrailer( false );

    //
    // Close the result-set and the statement.
    //
    rset.close();
    stmt.close();
  }

  /**
   * View a full-size photo
   */
  void viewPhoto()
    throws ServletException, IOException, SQLException
  {
    //
    // Write a common HTML header.
    //
    printPageHeader();

    //
    // Get the ID of the entry being viewed.
    //
    String expt_id = request.getParameter( "id" );
    //    expt_id = "3";

    //
    // Prepare and execute a SQL statement to fetch the entry's
    // description, expt. id and full-size image. The full-size
    // image is used to build the height and width attributes of
    // the image tag. Make sure the entry exists.
    //
    PreparedStatement stmt =
      conn.prepareStatement( "select expt_id, project_name, scientist_name, description, expt_date, file_name, image " + " from expt_img_tab_demo where expt_id = ?" );
    stmt.setString( 1, expt_id );
    OracleResultSet rset = (OracleResultSet)stmt.executeQuery();
    if ( !rset.next() )
    {
      out.println("expt_id" + expt_id);
      throw new ServletException( "row not found in table, expt_id:" + expt_id );

    }

    //
    // Get the thumb-nail OrdImage object from the result set so we
    // can build the height and width attributes in the image tag.
    //
    OrdImage img =
      (OrdImage)rset.getCustomDatum( 7, OrdImage.getFactory() );

    //
    // Display the data in a table, starting with the description and
    // expt id.
    //
    out.println( "<p><table>" );
    out.println( "<tr><td><b>Project Name:</b></td><td>" +
        rset.getString( 2 ) + "</td></tr>" );
    out.println( "<tr><td><b>Scientist Name:</b></td><td>" +
        rset.getString(3) + "</td></tr>" );
    out.println( "<tr><td><b>Description:</b></td><td>" +
        rset.getString(4) + "</td></tr>" );
    //      out.println( "<tr><td>" );
    //      printLinkMid( "?view_add=yes&id="+expt_id, "Add Annotation" );
    //  	out.println( "</td></tr>" );
    //  	out.println( "<tr><td> <br></td></tr>" );
    out.println( "<tr>" );
    out.println( "<form action=\"" + servletUri + "\"" );
    out.println( "      method=\"post\" enctype=\"multipart/form-data\">" );
    out.println( "<input type=\"hidden\" name=\"id\" value=" + expt_id + ">");
    out.println( "<td> <input type=\"submit\" name=\"add_ann\" value=\"Add Annotation\"> </td>");
    out.println( "<td> <input type=\"text\" name=\"new_desc\" size=\"60\"> </td>");
    out.println( " </form> ");
    out.println( "</tr>" );
    out.println( "<tr><td><b>Experiment date:</b></td><td>" +
        rset.getString(5) + "</td></tr>" );
    out.println( "<tr><td><b>Filename:</b></td><td>" +
        rset.getString(6) + "</td></tr>" );
    out.println( "<tr><td><b>Height:</b></td><td>" +
        img.getHeight() + "</td></tr>" );
    out.println( "<tr><td><b>Width:</b></td><td>" +
        img.getWidth() + "</td></tr>" );

    //
    // Build a URL to fetch the full-size image for this entry. If the
    // image format is not recognized, then don't generate the height
    // and width attributes for the image tag. Specifying the height
    // and width attributes allows the browser to render the page without
    // having to wait to retrieve the image to determine its size, but
    // are not required.
    //
    out.println( "<tr><td valign=\"top\"><b>Image:</b></td><td>" );
    out.print( "<img src=\"" + servletUri +
      "?view_media=image&id=" + expt_id + "\"" );
    if ( img.getHeight() > 0 && img.getWidth() > 0 )
    {
      out.print( " height=" + img.getHeight() +
          " width=" + img.getWidth() );
    }
    out.println( " border=1></td></tr>" );

    //
    // New - for rotate 
    //
    out.println("<tr><td>&nbsp;</td>");
    out.println("<td> <table>"); 
    out.println("<tr>");
    out.println("<td><form action='" + servletUri + "' method=get>");
    out.println("<input type=hidden name=id value=" + expt_id + ">");
    out.println("<input type=text name=rotate_angle size=5>&nbsp;");
    out.println("<input type=submit name=view_rotate value=Rotate></td>");
    //  out.println("</tr>");

    //
    // New - for contrast 
    //
    //  out.println("<tr><td>&nbsp;</td>");
    //  out.println("<td>&nbsp;</td>");
    out.println("<td><form action='" + servletUri + "' method=get>");
    out.println("<input type=hidden name=id value=" + expt_id + ">");
    out.println("<input type=text name=contrast_percent size=5>&nbsp;");
    out.println("<input type=submit name=view_contrast value=Contrast></td>");
    out.println("</tr>");

    //
    // New - for brightness 
    //
//  out.println("<tr><td>&nbsp;</td>");
    out.println("<tr>");
    out.println("<td><form action='" + servletUri + "' method=get>");
    out.println("<input type=hidden name=id value=" + expt_id + ">");
    out.println("<input type=text name=correct_brightness size=5>&nbsp;");
    out.println("<input type=submit name=view_brightness value=Brighten></td>");
    out.println("</tr>");


    out.println("</table>");
    out.println("</td>");
    out.println( "</table></p>" );

    //
    // Write the common HTML trailer
    //
    printPageTrailer( true );

    //
    // Close the result-set and the statement.
    //
    rset.close();
    stmt.close();
  }

  /**
   * Rotate a full-size image and retrieve from the database and
   * deliver it to the browser.
   */
  void viewRotateMedia() throws ServletException, IOException, SQLException
  {
    String sAngle = request.getParameter( "rotate_angle" );
    String sImageId = request.getParameter( "id" );

    if (sImageId==null)
      throw new ServletException("Need to specify an image for rotating");

    if (sAngle == null)
      sAngle = "0";

    try
    {
      Float fAngle = Float.valueOf(sAngle);
    }
    catch (NumberFormatException e)
    {
      sAngle = "0";
    }

    //
    // Prepare and execute a SQL statement to fetch the thumb-nail or
    // full-size image. Although the SELECT statement is built on-the-fly,
    // depending on whether the thumb-nail or full-size image is required,
    // this results in only 2 SQL statements being used (one for each
    // column), as the ID is specified as a parameter marker. Therefore,
    // the likelyhood is greater that the statements will be in the
    // server's cache.
    //
    OraclePreparedStatement stmt =
      (OraclePreparedStatement)conn.prepareStatement( "select image " + 
          " from expt_img_tab_demo where expt_id = ? " );
    stmt.setString( 1, sImageId);
    OracleResultSet rset = (OracleResultSet)stmt.executeQuery();

   OraclePreparedStatement stmt1 =
      (OraclePreparedStatement)conn.prepareStatement( "select image " + 
          " from temp_image_tab where expt_id = 1 for update" );
    //  stmt.setString( 1, sImageId);
    OracleResultSet rset1 = (OracleResultSet)stmt1.executeQuery();

    //
    // Fetch the row from the result set.
    //
    if ( rset.next() )
    {
      //
      // Get the OrdImage object from the result set.
      //
      OrdImage img =
        (OrdImage)rset.getCustomDatum(1, OrdImage.getFactory());
        // (OrdImage)rset.getORAData(1, OrdImage.getORADataFactory());
      rset1.next();
      OrdImage img1 =
        (OrdImage)rset.getCustomDatum(1, OrdImage.getFactory());
        // deprecated: use ...
        // (OrdImage)rset1.getORAData(1, OrdImage.getORADataFactory());

      //
      // Rotate the image
      //
      img.processCopy("rotate=" + sAngle,img1);

      //
      // Create an OrdHttpResponseHandler object, then use it to retrieve
      // the image from the database and deliver it to the browser.
      //
      OrdHttpResponseHandler handler =
        new OrdHttpResponseHandler( request, response );
      handler.sendImage( img1 );
    }
    else
    {
      //
      // Row not found, return a suitable error.
      //
      response.setStatus( response.SC_NOT_FOUND );
    }

    //
    // Close the result-set and the statement.
    //
    rset.close();
    stmt.close();
  }


  /*****************************************************************************
   * Enhance a full-size image and retrieve from the database and
   * deliver it to the browser.
   */
  void viewContrastMedia() throws ServletException, IOException, SQLException
  {
    String sContrast = request.getParameter( "contrast_percent" );
    String sImageId = request.getParameter( "id" );

    if (sImageId==null)
      throw new ServletException("Need to specify an image for adjusting contrast");

    if (sContrast== null)
      sContrast = "0";

    try
    {
      Float fContrast = Float.valueOf(sContrast);
    }
    catch (NumberFormatException e)
    {
      sContrast = "0";
    }

    //
    // Prepare and execute a SQL statement to fetch the thumb-nail or
    // full-size image. Although the SELECT statement is built on-the-fly,
    // depending on whether the thumb-nail or full-size image is required,
    // this results in only 2 SQL statements being used (one for each
    // column), as the ID is specified as a parameter marker. Therefore,
    // the likelyhood is greater that the statements will be in the
    // server's cache.
    //
    OraclePreparedStatement stmt =
      (OraclePreparedStatement)conn.prepareStatement( "select image " + 
          " from expt_img_tab_demo where expt_id = ? " );
    stmt.setString( 1, sImageId);
    OracleResultSet rset = (OracleResultSet)stmt.executeQuery();

    OraclePreparedStatement stmt1 =
      (OraclePreparedStatement)conn.prepareStatement( "select image " + 
          " from temp_image_tab where expt_id = 1 for update" );
//    stmt.setString( 1, sImageId);
    OracleResultSet rset1 = (OracleResultSet)stmt1.executeQuery();

    //
    // Fetch the row from the result set.
    //
    if ( rset.next() )
    {
      //
      // Get the OrdImage object from the result set.
      //
       OrdImage img =
       (OrdImage)rset.getCustomDatum(1, OrdImage.getFactory());
//        (OrdImage)rset.getORAData(1, OrdImage.getORADataFactory());
      rset1.next();
      OrdImage img1 =
       (OrdImage)rset.getCustomDatum(1, OrdImage.getFactory());
//        (OrdImage)rset1.getORAData(1, OrdImage.getORADataFactory());

      //
      // Rotate the image
      //
      img.processCopy("contrast=" + sContrast,img1);

      //
      // Create an OrdHttpResponseHandler object, then use it to retrieve
      // the image from the database and deliver it to the browser.
      //
      OrdHttpResponseHandler handler =
        new OrdHttpResponseHandler( request, response );
      handler.sendImage( img1 );
    }
    else
    {
      //
      // Row not found, return a suitable error.
      //
      response.setStatus( response.SC_NOT_FOUND );
    }

    //
    // Close the result-set and the statement.
    //
    rset.close();
    stmt.close();
  }


  void viewBrightnessMedia() throws ServletException, IOException, SQLException
  {
    String sBrightness = request.getParameter( "correct_brightness" );
    String sImageId = request.getParameter( "id" );

    if (sImageId==null)
      throw new ServletException("Need to specify an image for adjusting contrast");

    if (sBrightness == null)
      sBrightness = "0";

    try
    {
      Float fBrightness = Float.valueOf(sBrightness);
    }
    catch (NumberFormatException e)
    {
      sBrightness = "0";
    }

    //
    // Prepare and execute a SQL statement to fetch the thumb-nail or
    // full-size image. Although the SELECT statement is built on-the-fly,
    // depending on whether the thumb-nail or full-size image is required,
    // this results in only 2 SQL statements being used (one for each
    // column), as the ID is specified as a parameter marker. Therefore,
    // the likelyhood is greater that the statements will be in the
    // server's cache.
    //
    OraclePreparedStatement stmt =
      (OraclePreparedStatement)conn.prepareStatement( "select image " + 
          " from expt_img_tab_demo where expt_id = ? " );
    stmt.setString( 1, sImageId);
    OracleResultSet rset = (OracleResultSet)stmt.executeQuery();

    OraclePreparedStatement stmt1 =
      (OraclePreparedStatement)conn.prepareStatement( "select image " + 
          " from temp_image_tab where expt_id = 1 for update" );
//  stmt.setString( 1, sImageId);
    OracleResultSet rset1 = (OracleResultSet)stmt1.executeQuery();

    //
    // Fetch the row from the result set.
    //
    if ( rset.next() )
    {
      //
      // Get the OrdImage object from the result set.
      //
      OrdImage img =
        (OrdImage)rset.getCustomDatum(1, OrdImage.getFactory());
//      (OrdImage)rset.getORAData(1, OrdImage.getORADataFactory());

      rset1.next();
      OrdImage img1 =
        (OrdImage)rset.getCustomDatum(1, OrdImage.getFactory());
//      (OrdImage)rset1.getORAData(1, OrdImage.getORADataFactory());

      //
      // Correct brightness of the image
      //
      img.processCopy("gamma=" + sBrightness,img1);

      //
      // Create an OrdHttpResponseHandler object, then use it to retrieve
      // the image from the database and deliver it to the browser.
      //
      OrdHttpResponseHandler handler =
        new OrdHttpResponseHandler( request, response );
      handler.sendImage( img1 );
    }
    else
    {
      //
      // Row not found, return a suitable error.
      //
      response.setStatus( response.SC_NOT_FOUND );
    }

    //
    // Close the result-set and the statement.
    //
    rset.close();
    stmt.close();
  }


  /*****************************************************************************
   * Retrieve a thumb-nail or full-size image from the database and
   * deliver it to the browser.
   */
  void viewMedia( String media ) throws ServletException, IOException, SQLException
  {
    //
    // Prepare and execute a SQL statement to fetch the thumb-nail or
    // full-size image. Although the SELECT statement is built on-the-fly,
    // depending on whether the thumb-nail or full-size image is required,
    // this results in only 2 SQL statements being used (one for each
    // column), as the ID is specified as a parameter marker. Therefore,
    // the likelyhood is greater that the statements will be in the
    // server's cache.
    //
    PreparedStatement stmt =
      conn.prepareStatement( "select " + media +
          " from expt_img_tab_demo where expt_id = ?" );
    stmt.setString( 1, request.getParameter( "id" ) );
    OracleResultSet rset = (OracleResultSet)stmt.executeQuery();

    //
    // Fetch the row from the result set.
    //
    if ( rset.next() )
    {
      //
      // Get the OrdImage object from the result set.
      //
      OrdImage img =
        (OrdImage)rset.getCustomDatum(1, OrdImage.getFactory());

      //
      // Create an OrdHttpResponseHandler object, then use it to retrieve
      // the image from the database and deliver it to the browser.
      //
      OrdHttpResponseHandler handler =
        new OrdHttpResponseHandler( request, response );
      handler.sendImage( img );
    }
    else
    {
      //
      // Row not found, return a suitable error.
      //
      response.setStatus( response.SC_NOT_FOUND );
    }

    //
    // Close the result-set and the statement.
    //
    rset.close();
    stmt.close();
  }


  /**
   * The HTML upload form as a static String.
   */
  private static final String ANN_FORM_FIELDS =
    "<table border=\"0\" width=\"100%\" cellspacing=\"0\">\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td colspan=\"2\">&nbsp;</td>\r\n" +
    " </tr>\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td valign=\"top\"><b>Description:</b>\r\n" +
    "  <td><input type=\"text\" name=\"new_desc\" size=\"60\">\r\n" +
    "      <br><font size=\"-1\">(Necessary) </font></td>\r\n" +
    " </tr>\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td colspan=\"2\"><input type=\"submit\" name=\"add_ann\" value=\"Add Annotation\" ></td>\r\n" +
    " </tr>\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td colspan=\"2\">&nbsp;</td></tr>\r\n" +
    "</table>\r\n";

  /**
   * The HTML upload form as a static String.
   */
  private static final String UPLOAD_FORM_FIELDS =
    "<table border=\"0\" width=\"100%\" cellspacing=\"0\">\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td colspan=\"2\">&nbsp;</td>\r\n" +
    " </tr>\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td valign=\"top\"><b>Project Name:</b>\r\n" +
    "  <td><input type=\"text\" name=\"Project_Name\" length=\"40\">\r\n" +
    "      <br><font size=\"-1\">(Necessary) </font></td>\r\n" +
    " </tr>\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td valign=\"top\"><b>Scientist Name:</b>\r\n" +
    "  <td><input type=\"text\" name=\"Scientist_Name\" length=\"40\">\r\n" +
    "      <br><font size=\"-1\"> (Necessary)  </font></td>\r\n" +
    " </tr>\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td valign=\"top\"><b>Description:</b>\r\n" +
    "  <td><input type=\"text\" name=\"Description\" length=\"40\">\r\n" +
    "      <br><font size=\"-1\">(Optional)   </font></td>\r\n" +
    " </tr>\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td valign=\"top\"><b>Experiment Date:</b>\r\n" +
    "  <td><input type=\"text\" name=\"Experiment_Day\" size=\"2\">\r" +
    "      <input type=\"text\" name=\"Experiment_Month\" size=\"3\">\r" +
    "      <input type=\"text\" name=\"Experiment_Year\" size=\"4\">\r\n" +
    "      <br><font size=\"-1\">(dd-mon-yyyy)</font></td>\r\n" +
    " </tr>\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td valign=\"top\"><b>File name:</b></td>\r\n" +
    "  <td><input type=\"file\" name=\"photo\">\r\n" +
    "      <br><font size=\"-1\">(e.g., EColi2.gif)</font></td>\r\n" +
    " </tr>\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td colspan=\"2\">&nbsp;</td>\r\n" +
    " </tr>\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td colspan=\"2\"><input type=\"submit\" name=\"insert_image\" value=\"Insert image\" ></td>\r\n" +
    " </tr>\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td colspan=\"2\">&nbsp;</td></tr>\r\n" +
    "</table>\r\n";

    // TODO: PREDELAT!!!
  /**
   * The HTML query form as a static String.
   */
  private static final String QUERY_FORM_FIELDS =
    "<table border=\"0\" width=\"100%\" cellspacing=\"0\">\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td colspan=\"2\">&nbs;</td>\r\n" +
    " </tr>\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td valign=\"top\"><b>Project Name:</b> </td> \r\n" +
    "  <td><input type=\"text\" name=\"Project_Name\" length=\"40\">\r\n" +
    "      <br><font size=\"-1\">(e.g., lymphoma, 2-d gels, medical, microarrays)</font><tr></td>\r\n" +
    " </tr>\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td colspan=\"2\"><input type=\"submit\" name=\"query_table\" value=\"Query table\" ></td>\r\n" +
    " </tr>\r\n" +
    " <tr bgcolor=\"#f7f7e7\">\r\n" +
    "  <td colspan=\"2\">&nbsp;</td></tr>\r\n" +
    "</table>\r\n";

  /*****************************************************************************
   * Display an HTML form that allows annotations to be added to an image.
   */
  void viewAddAnnForm() throws ServletException, IOException
  {
    //
    // Output the page header, the form and the page trailer.
    //
    viewAddAnnForm( null );
  }

  void viewAddAnnForm( String errorMessage ) throws ServletException, IOException
  {
    //
    // Output the page header, the form and the page trailer.
    //
    printPageHeader();
    if ( errorMessage != null )
    {
      printMessage( "Error message", errorMessage );
    }
    out.println( "<p><form action=\"" + servletUri + "\"" );
    out.println( "      method=\"post\" enctype=\"multipart/form-data\">" );
    out.println( ANN_FORM_FIELDS );
    out.println( "</form></p>" );
    printPageTrailer( true );
  }

  /*****************************************************************************
   * Display an HTML form that allows a new entry to be added to the album.
   */
  void viewUploadForm() throws ServletException, IOException
  {

    // TODO: odtud...
    // Output the page header, the form and the page trailer.
    //
    viewUploadForm( null );
  }

  /*****************************************************************************
   * Display an HTML form that allows a new entry to be added to the album.
   */
  void viewUploadForm( String errorMessage ) throws ServletException, IOException
  {
    //
    // Output the page header, the form and the page trailer.
    //
    printPageHeader("// TODO: prozkoumat doGet, nakopirovat do vyhledavani (+ opravit chybu)");
    if ( errorMessage != null )
    {
      printMessage( "Error message", errorMessage );
    }
    out.println( "<p><form action=\"" + servletUri + "\"" );
    out.println( "      method=\"post\" enctype=\"multipart/form-data\">" );
    out.println( UPLOAD_FORM_FIELDS );
    out.println( "</form></p>" );
    printPageTrailer( true );
  }

  /*****************************************************************************
   * Display an HTML form that allows a new entry to be added to the album.
   */
  void viewQueryForm() throws ServletException, IOException
  {
    // TODO: tudy dale
    // Output the page header, the form and the page trailer.
    //
    viewQueryForm( null );
  }

  void viewQueryForm( String errorMessage ) throws ServletException, IOException
  {
    //
    // Output the page header, the form and the page trailer.
    //
    printPageHeader("// TODO: tohle predelat jako upload (Insert Image)");
    if ( errorMessage != null )
    {
      printMessage( "Error message", errorMessage );
    }
    out.println( "<p><form action=\"" + servletUri + "\"" );
    out.println( "      method=\"post\" enctype=\"multipart/form-data\">" );
    out.println( QUERY_FORM_FIELDS );
    out.println( "</form></p>" );
    printPageTrailer( true );
  }

  /*****************************************************************************
   * This method  gets the query parameters and queries the database.
   */
  void queryImages(OrdHttpUploadFormData formData) throws ServletException, IOException, SQLException
  {
    int rowCount = 0;

    //
    // Create an OrdHttpUploadFormData object.
    //
    // OrdHttpUploadFormData formData = new OrdHttpUploadFormData( request );

    //
    // Display the upload form if not an upload request.
    //
    if ( !formData.isUploadRequest() )
    {
      viewQueryForm();
      return;
    }
    //
    // Parse the multipart/form-data message.
    //
    //      formData.parseFormData();

    // Write a common HTML header
    printPageHeader("// TODO: ulozit uploadovany obrazek, vyhledat podobne a seradit dle vzdalenosti");

    // TODO: viz. zadani cviceni 1
    // Melli
    //
    // Get the description, expt. id and photo.
    //
    String projName = formData.getParameter( "Project_Name" );

    PreparedStatement stmt = 
      conn.prepareStatement( "select expt_id,thumb,description " +
          " from expt_img_tab_demo " +
          " where project_name = ? " + 
          " order by expt_id" );
    stmt.setString( 1, projName);
    OracleResultSet rset = (OracleResultSet)stmt.executeQuery();


    // not TODO: tohle uz nechat
    //
    // Display the thumb-nail images in a table.
    //
    out.println( "<p><table cellpadding=\"3\" cellspacing=\"0\"" +
        " border=\"1\" width=\"100%\">" );
    out.println( "<tr bgcolor=\"#336699\">" );
    //      out.println( "<th><font color=\"#ffffff\">Expt id</font></th>" );
    out.println( "<th><font color=\"#ffffff\">Image</font></th>" );
    out.println( "<th><font color=\"#ffffff\">Description</font></th></tr>" );

    //
    // For each entry...
    //
    while ( rset.next() )
    {
      //
      // Increment the row count so we know if there are any entries
      // in the album.
      //
      rowCount++;

      //
      // Get the entry ID for building URLs.
      //
      String expt_id = rset.getString( 1 );

      //
      // Start the table row and display the photo description and
      // expt. id.
      //
      out.println( "<tr>" );
      //	    out.println( "<td>" + expt_id + "</td>" );

      //
      // Get the thumb-nail OrdImage object from the result set so we
      // can build the height and width attributes in the image tag.
      //
      OrdImage img =
        (OrdImage)rset.getCustomDatum( 2, OrdImage.getFactory() );

      //
      // Display the thumb-nail image as an anchor tag which can be used
      // to display the full-size image. If the image format isn't
      // supported by interMedia, then a thumb-nail wouldn't have been
      // produced when the image was uploaded, so use the text "[view
      // image]" instead of the thumb-nail.
      //
      out.print( "<td><a href=\"" + servletUri +
        "?view_entry=yes&id=" + expt_id + "\">" );
      if ( img.getContentLength() > 0 )
      {
        out.print( "<img src=\"" + servletUri +
          "?view_media=thumb&id=" + expt_id + "\"" +
          " height=" + img.getHeight() +
          " width=" + img.getWidth() +
          " border=1>" );
      }
      else
      {
        out.print( "[view image]" );
      }
      out.println( "</a></td>" );

      out.println( "<td>" + rset.getString( 3 ) + "</td>" );
      out.println( "</tr>" );
    }

    //
    // Display a message if the album is empty
    //
    out.println( "<tr><td colspan=\"3\" align=\"center\">" +
        "<font color=\"#336699\">" );
    if ( rowCount == 0 )
    {
      out.println( "<b><i>No images of that experiment type</i></b>" );
    }
    else
    {
      out.println( "<b><i>Select the thumb-nail to view the" +
          " full-size image</i></b>" );
    }
    out.println( "</font></td></tr>" );
    out.println( "</table>" );

    //
    // Display an anchor tag that will display the new entry form.
    //
    // printLink( "?view_form=yes", "Insert new image" );

    //
    // Write the common HTML trailer
    //
    printPageTrailer( true );

    //
    // Close the result-set and the statement.
    //
    rset.close();
    stmt.close();
  }

  /*****************************************************************************
   * This method is called by the doPost method to insert a new photo
   * into the album, creating a thumb-nail image at the same time.
   */
  void insertNewPhoto(OrdHttpUploadFormData formData) throws ServletException, IOException, SQLException
  {
    //
    // Create an OrdHttpUploadFormData object.
    //
    //OrdHttpUploadFormData formData = new OrdHttpUploadFormData( request );

    //
    // Display the upload form if not an upload request.
    //
    if ( !formData.isUploadRequest() )
    {
      viewUploadForm();
      return;
    }

    //
    // Parse the multipart/form-data message.
    //

    // formData.parseFormData();
    Enumeration nameList = formData.getParameterNames();

    while (nameList.hasMoreElements())
    {
      String name = (String) nameList.nextElement();
      System.out.println(name);
    }

    //
    // Get the description, expt. id and photo.
    //
    String projName = formData.getParameter( "Project_Name" );
    String sciName = formData.getParameter( "Scientist_Name" );
    String desc = formData.getParameter( "Description" );
    String exptDay = formData.getParameter( "Experiment_Day" );
    Integer exptDayI = new Integer(exptDay);
    String exptMonth = formData.getParameter( "Experiment_Month" );

    //	Integer exptMonthI = new Integer(exptMonth);
    String exptYear = formData.getParameter( "Experiment_Year" );
    Integer exptYearI = new Integer(exptYear);
    // watch this
    OrdHttpUploadFile photo = formData.getFileParameter( "photo" );
    String fileName = photo.getSimpleFileName();

    String exptDate = exptDay + "-" + exptMonth + "-" + exptYear;
    System.out.println(projName);
    System.out.println(sciName);
    System.out.println(desc);
    System.out.println(fileName);
    System.out.println(exptYearI);
    System.out.println(exptDayI);
    System.out.println(exptDate);

    //  System.out.println(exptDate);
    //	System.out.println(photo.getOriginalFileName());
    //	System.out.println(photo.getSimpleFileName());

    //
    // Make sure a valid image file was provided.
    //
    if ( photo == null ||
        photo.getOriginalFileName() == null ||
        photo.getContentLength() == 0 )
    {
      viewUploadForm( "Please supply a valid image file." );
      return;
    }

    //
    // Use the file name if there's no description.
    //
    //
    if ( desc == null || desc.length() == 0 )
    {
      desc = photo.getSimpleFileName();
    }

    //
    // We're going to be updating the database and writing to LOBs, so
    // make sure auto-commit is disabled.
    //
    conn.setAutoCommit( false );

    //
    // Get a value for the ID column of the new row
    // TODO: opravit chybu
    // * nejlepe sekvenci
    // * hure vlozit nejake id do tabulky expt_img_tab_demo (bacha na soubezny pristup)
    //
    OraclePreparedStatement stmt =
       (OraclePreparedStatement)conn.prepareStatement(
                           "select max(expt_id)+1 from expt_img_tab_demo" );
     OracleResultSet rset = (OracleResultSet)stmt.executeQuery();
     if ( !rset.next() )
     {
       throw new ServletException( "new ID not found" );
     }
     String expt_id = rset.getString( 1 );

     rset.close();
     stmt.close();

    //
    // Prepare and execute a SQL statement to insert the new row.
    //
    stmt = (OraclePreparedStatement)conn.prepareStatement(

        "insert into expt_img_tab_demo(expt_id,project_name,scientist_name,description,expt_date,file_name,image,thumb) " +
        " values (?,?,?,?,?,?,ORDSYS.ORDImage.init(),ORDSYS.ORDImage.init())" );

    stmt.setString( 1, expt_id );
    stmt.setString( 2, projName);
    stmt.setString( 3, sciName);
    stmt.setString( 4, desc);
    stmt.setString( 5, exptDate);
    stmt.setString( 6, fileName);

    stmt.executeUpdate();
    stmt.close();

    //
    // Prepare and execute a SQL statement to fetch the initialized
    // full-size and thumb-nail image objects from the table.
    //

    stmt = (OraclePreparedStatement)conn.prepareStatement(
        "select image,thumb from expt_img_tab_demo where expt_id = ? for update" );
    stmt.setString( 1, expt_id );
    rset = (OracleResultSet)stmt.executeQuery();
    if ( !rset.next() )
    {
      throw new ServletException( "new row not found in table" );
    }

    OrdImage image =
      (OrdImage)rset.getCustomDatum( 1, OrdImage.getFactory());
    OrdImage thumb =
      (OrdImage)rset.getCustomDatum( 2, OrdImage.getFactory());
    rset.close();
    stmt.close();

    //
    // Load the photo into the database and set the properties.
    //
    photo.loadImage( image );

    //
    // Some image formats are supported by interMedia but may not be able
    // to be displayed in-line by a browser. The BMP format is one example.
    // Convert the image to a GIF or JPEG based on number of colors in the
    // image.
    //

    if ( image.getContentFormat() != null &&
        image.getMimeType().indexOf( "bmp" ) > 0 )
    {
      try
      {
        image.process( "fileFormat=" +
            getPreferredFormat( image.getContentFormat() ) );
      }
      catch ( SQLException e )
      {
      }
    }

    //
    // Try to copy the full-size image and process it to create the
    // thumb-nail. This may not be possible if the image format is
    // not recognized.
    //
    try
    {
      image.processCopy( "maxScale=50,50", thumb );
    }
    catch ( SQLException e )
    {
      thumb.deleteContent();
      thumb.setContentLength( 0 );
    }

    //
    // Prepare and execute a SQL statement to update the full-size and
    // thumb-nail images in the database.
    //
    stmt = (OraclePreparedStatement)conn.prepareStatement(
        "update expt_img_tab_demo set image = ?, thumb = ? where expt_id = ?" );
    stmt.setCustomDatum( 1, image );
    stmt.setCustomDatum( 2, thumb );
    stmt.setString( 3, expt_id );
    stmt.execute();
    stmt.close();

    //
    // Commit the changes.
    //
    conn.commit();

    //
    // Direct the browser to display the main page.
    //
    // "<meta http-equiv=\"refresh\" content=\"2 ;url=" + servletUri + "\">"
    printPageHeader( " // TODO: opravit chybu zde, prozkoumat doPost, nakopirovat do vyhledavani" );
    printMessage( "Photo successfully uploaded into photo album",
        "Please click on the link below or" +
        " wait for the browser to refresh the page." );
    printPageTrailer( true );

  }

  /*****************************************************************************
   * This method is called by the doPost method to update Annotations
   * of an image (updates Description column
   */
  void addAnnotations(OrdHttpUploadFormData formData) throws ServletException, IOException, SQLException
  {
    // Display Annotation add form if not an annotation add request
    if ( !formData.isUploadRequest() )
    {
      viewAddAnnForm();   
      return;
    }
    // Write a common HTML header
    printPageHeader();

    // Get the annotation
    String newDesc = formData.getParameter( "new_desc" );
    String upd_expt_id = formData.getParameter( "id" );

    PreparedStatement stmt =
      conn.prepareStatement( "Update expt_img_tab_demo set description = description || ' ' || ? where expt_id = ?");

    stmt.setString(1,newDesc);
    stmt.setString(2,upd_expt_id);
    stmt.executeUpdate();
    System.out.println(upd_expt_id);
    System.out.println(newDesc);

    printPageHeader(
        "<meta http-equiv=\"refresh\" content=\"2 ;url=" + servletUri + "\">" );
    printMessage( "Photo successfully uploaded into photo album",
        "Please click on the link below or" +
       " wait for the browser to refresh the page." );
    printPageTrailer( true );

  }


  /*****************************************************************************
   * Private function to return the preferred file format to which
   * image formats not supported by a browser should be converted.
   */
  private String getPreferredFormat( String contentFormat )
  {
    //
    // Image content format strings have the following format:
    //   <#bits><format>
    //   MONOCHROME
    //
    int numDigits = 0;
    while ( Character.isDigit( contentFormat.charAt( numDigits ) ) )
    {
      numDigits++;
    }

    //
    // Images with more than 8 bits of color can be converted to the JPEG
    // format without significant discernible loss of quality.
    //
    if ( numDigits > 0 )
    {
      if ( Integer.parseInt( contentFormat.substring( 0, numDigits ) ) > 8 )
      {
        return "JFIF";
      }
    }

    //
    // Images with 8 bits of color or less are best converted to the GIF
    // format to retain the quality.
    //
    return "GIFF";
  }

  /*****************************************************************************
   * Display an HTML header that is common to all HTML responses.
   */
  private void printPageHeader() throws ServletException, IOException
  {
    printPageHeader( null );
  }

  /**
   * Display an HTML header that is common to all HTML responses.
   */
  private void printPageHeader( String extraHeader ) throws ServletException, IOException
  {
    //
    // Get the PrintWriter output stream.
    //
    out = response.getWriter();

    //
    // Start the response.
    //
    response.setContentType( "text/html" );
    out.println( "<html>" );
    out.println( "<head>" );
    out.println( "<title>interMedia Java Servlet Demo</title>" );

    if ( extraHeader != null )
    {
      out.println( extraHeader );
    }
    out.println( "</head>" );
    out.println( "<body>" );
    out.println( "<table border=\"0\" width=\"100%\">" );
    out.println( "<tr>" );
    out.println( " <td colspan=\"2\" bgcolor=\"#f7f7e7\" align=\"center\">" );
    out.println( "  <font size=\"+2\">" );
    out.println( "   <i>inter</i>Media Java Servlet Demo" );
    out.println( "  </font>" );
    out.println( " </td>" );
    out.println( "</tr>" );
    out.println( "</table>" );
  }

  /**
   * Display an HTML trailer that is common to all HTML responses.
   */
  private void printPageTrailer( boolean displayReturnUrl ) throws ServletException, IOException
  {
    if ( displayReturnUrl )
    {
      printLink( "", "Return to main display page" );
    }
    out.println( "</body></html>" );
  }

  /**
   * Print a message
   */
  private void printMessage( String heading, String message )
  {
    printHeading( heading );
    out.println( "<p>" + message + "</p>" );
  }

  /**
   * Print a heading on a page
   */
  private void printHeading( String heading )
  {
    out.println( "<p><font size=3 color=\"#336699\">" );
    out.println( "<b>" + heading + "</b></font><hr size=1></p>" );
  }

  /**
   * Output an anchor tag in a standard format.
   */
  private void printLink( String queryString, String text )
  {
    out.println( "<p><table width=\"100%\"><tr bgcolor=\"#f7f7e7\">" );
    out.println( " <td colspan=\"3\" align=\"center\">" );
    out.println( "  <a href=\"" + servletUri + queryString + "\">" +
        text + "</a>" );
    out.println( " </td>" );
    out.println( "</tr></table></p>" );
  }

  private void printLinkMid( String queryString, String text)
  {
    out.println("  <a href=\"" + servletUri + queryString + "\">" +
        text + "</a>" );
  }
}


